/******************************************************************************/
/*   Copyright (C) 2006 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "deal1type.h"
#include "deal1ext.h"
/******************************************************************************/
int Simulation()
{
 for(Frame_Cnt=ZERO;Frame_Cnt<MAX_FRAMES;Frame_Cnt++)
 {
  Init_Frame();
  Get_Frame();                                            /* Go to deal1sym.c */
  Get_Tx_Flt();
  Get_Tx_Flt_Op();
  Get_Sync();                                            /* Go to deal1sync.c */
  Sample_MF_Op();
 }
 return(0);
}
/******************************************************************************/
/*                    Initialize the variables in a frame.
*******************************************************************************/
int Init_Frame()
{
 double erand48();
 if(RAND_TIM == ONE)
  Rand_Tx_Delay=(double)TX_DELAY+(double)TX_IP_FAC*erand48(N1_Store);
 else
  Rand_Tx_Delay=(double)TX_DELAY;
 if(RAND_PHASE == ONE)
  Initial_Phase=Theta=2.0*M_PI*erand48(N1_Store);
 else
  Initial_Phase=Theta=0.25*M_PI;
 Tx_Samp_Cnt=Rx_Samp_Cnt=ZERO;
 Tx_Reg_Init();
 return(0);
}
/******************************************************************************/
/*                        Get the transmit filter.
*******************************************************************************/
int Get_Tx_Flt()
{
 Get_RRC(TX_FLT_SIZE,Rand_Tx_Delay,TX_SAMP_FREQ,Tx_Flt); /* Go to deal1util.c */
 Normalize_Tx_Filter();
 return(0);
}
/******************************************************************************/
/*                      Normalize the transmit filter.
*******************************************************************************/
int Normalize_Tx_Filter()
{
 int cnt;
 double energy,sqrt_energy;
 energy=0.0;
 for(cnt=ZERO;cnt<TX_FLT_SIZE;cnt++)
  energy=energy+Tx_Flt[cnt]*Tx_Flt[cnt];
 sqrt_energy=sqrt(energy);
 for(cnt=ZERO;cnt<TX_FLT_SIZE;cnt++)
  Tx_Flt[cnt]=Tx_Flt[cnt]/sqrt_energy;
 return(0);
}
/******************************************************************************/
/*                       Get the transmit filter output.
*******************************************************************************/
int Get_Tx_Flt_Op()
{
 int cnt1,bit0,bit1,ip_cnt;
 double re_sym,im_sym;
 for(cnt1=ZERO;cnt1<FRAME_SIZE;cnt1++)
 {
  bit0=Sym[cnt1]&ONE;
  bit1=Sym[cnt1]>>ONE;
  re_sym=Map[bit0];
  im_sym=Map[bit1];
  for(ip_cnt=ZERO;ip_cnt<TX_IP_FAC;ip_cnt++)
  {
   Compute_Tx_Op(ip_cnt,re_sym,im_sym);
   re_sym=im_sym=0.0;
  }
 }
 return(0);
}
/******************************************************************************/
/*             Compute the current output of the transmit filter.
*******************************************************************************/
int Compute_Tx_Op(ip_cnt,re_sym,im_sym)
int ip_cnt;
double re_sym,im_sym;
{
 int cnt;
 Cmplx_Shift_from_Zero(Re_Tx_Reg,Im_Tx_Reg,TX_FLT_SIZE);
 Re_Tx_Reg[ZERO]=re_sym;
 Im_Tx_Reg[ZERO]=im_sym;
 Re_Tx_Op=Im_Tx_Op=0.0;
 for(cnt=ip_cnt;cnt<TX_FLT_SIZE;cnt=cnt+TX_IP_FAC)
 {
  Re_Tx_Op=Re_Tx_Op+Tx_Flt[cnt]*Re_Tx_Reg[cnt];
  Im_Tx_Op=Im_Tx_Op+Tx_Flt[cnt]*Im_Tx_Reg[cnt];
 }
 Modulate();
 Demodulate();
 Get_MF_Op();
 return(0);
}
/******************************************************************************/
/*                      Modulate the baseband signal.
*******************************************************************************/
int Modulate()
{
 double erand48();
 if(Frame_Cnt == ZERO)
  fprintf(fp2,"%lf\n",sqrt(Re_Tx_Op*Re_Tx_Op+Im_Tx_Op*Im_Tx_Op));
 Passband_Signal=Re_Tx_Op*cos(Theta)-Im_Tx_Op*sin(Theta);
 N1_Val[URADIUS]=erand48(N1_Store);
 N1_Val[UTHETA]=erand48(N2_Store);
 My_Gauss(Variance,N1_Val);                              /* Go to deal1util.c */
 Passband_Signal=Passband_Signal+N1_Val[GRAND_0];
 Theta=Theta+0.5*M_PI;
 if(Theta > (2.0*M_PI))
  Theta=Theta-2.0*M_PI;
 return(0);
}
/******************************************************************************/
/*                      Demodulate the passband signal.
*******************************************************************************/
int Demodulate()
{
 Re_LO_Op=Passband_Signal*Re_LO[Tx_Samp_Cnt%LO_PERIOD];
 Im_LO_Op=Passband_Signal*Im_LO[Tx_Samp_Cnt%LO_PERIOD];
 Tx_Samp_Cnt++;
 return(0);
}
/******************************************************************************/
/*                            Get the MF output.
*******************************************************************************/
int Get_MF_Op()
{
 int cnt;
 for(cnt=ZERO;cnt<RX_IP_FAC;cnt++)
 {
  Cmplx_Shift_from_Zero(Re_MF_Reg,Im_MF_Reg,MF_SIZE);    /* Go to deal1util.c */
  Re_MF_Reg[ZERO]=Re_LO_Op;
  Im_MF_Reg[ZERO]=Im_LO_Op;
  Compute_MF_Op(cnt);
  Re_LO_Op=Im_LO_Op=0.0;
 }
 return(0);
}
/******************************************************************************/
/*                       Compute the current MF output.
*******************************************************************************/
int Compute_MF_Op(ip_cnt)
int ip_cnt;
{
 int cnt;
 if(Rx_Samp_Cnt == RX_SAMP_SIZE)
  Error_Mes(3);
 Re_MF_Op[Rx_Samp_Cnt]=Im_MF_Op[Rx_Samp_Cnt]=0.0;
 for(cnt=ip_cnt;cnt<MF_SIZE;cnt=cnt+RX_IP_FAC)
 {
  Re_MF_Op[Rx_Samp_Cnt]=Re_MF_Op[Rx_Samp_Cnt]+MF[cnt]*Re_MF_Reg[cnt];
  Im_MF_Op[Rx_Samp_Cnt]=Im_MF_Op[Rx_Samp_Cnt]+MF[cnt]*Im_MF_Reg[cnt];
 }
 Rx_Samp_Cnt++;
 return(0);
}
/******************************************************************************/
/*                             Sample the MF output.
*******************************************************************************/
int Sample_MF_Op()
{
 int cnt;
 Samp_Instant=Samp_Instant+PRE_AMB_SIZE*RX_SAMP_PER_SYM;
 for(cnt=ZERO;cnt<SYM_SIZE;cnt++)
 {
  Process_Rx_Samp(cnt);
  Samp_Instant=Samp_Instant+RX_SAMP_PER_SYM;
 }
 return(0);
}
/******************************************************************************/
/*                   Get the received samples from encoder 1.
*******************************************************************************/
int Process_Rx_Samp(cnt)
int cnt;
{
 Derotate();
 Get_Errors(cnt);
 return(0);
}
/******************************************************************************/
/*                            Derotate sample.
*******************************************************************************/
int Derotate()
{
 double re_tmp,im_tmp,cost,sint;
 if(Samp_Instant >= RX_SAMP_SIZE)
  Error_Mes(3);
 re_tmp=Re_MF_Op[Samp_Instant];
 im_tmp=Im_MF_Op[Samp_Instant];
 cost=cos(Est_Phase);
 sint=sin(Est_Phase);
 Rx_Re_Sym=re_tmp*cost+im_tmp*sint;
 Rx_Im_Sym=im_tmp*cost-re_tmp*sint;
 return(0);
}
/******************************************************************************/
/*                              Get the errors.
*******************************************************************************/
int Get_Errors(cnt)
int cnt;
{
 int bit0,bit1,rx_bit0,rx_bit1,cnt1;
 cnt1=cnt+PRE_AMB_SIZE;
 rx_bit0=rx_bit1=ZERO;
 bit0=Sym[cnt1]&ONE;
 bit1=Sym[cnt1]>>ONE;
 if(Rx_Re_Sym < FRAC_ZERO)
  rx_bit0=ONE;
 if(Rx_Im_Sym < FRAC_ZERO)
  rx_bit1=ONE;
 if(bit0 != rx_bit0)
  Ber++;
 if(bit1 != rx_bit1)
  Ber++;
 return(0);
}
/******************************************************************************/
