/******************************************************************************/
/*   Copyright (C) 2006 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "deal10type.h"
#include "deal10ext.h"
/******************************************************************************/
int Get_Sync()
{
 int cnt1,min,max;
 Init_Sync();
 for(cnt1=MIN_TIM;cnt1<MAX_TIM;cnt1++)
 {
  Compute_Tim_Correlation(cnt1);
  Check_Tim_Correlation(cnt1);
 }
 if((RAND_TIM == ZERO) && (fabs(CLOCK_ERR) < ERROR))
  Compute_Tim_Var();
 Compute_Carrier_Phase();
 Compute_Amplitude();
 Compute_Noise_Variance();
 Print_Sync_Results();
 return(0);
}
/******************************************************************************/
/*                 Initialize the synchronization variables.
*******************************************************************************/
int Init_Sync()
{
 Tim_Max_Mag=MIN_MAG;
 Mean_Corr=0.0;
 Mean_Corr_Cnt=ZERO;
 return(0);
}
/******************************************************************************/
/*                      Compute the current correlation.
*******************************************************************************/
int Compute_Tim_Correlation(cnt1)
int cnt1;
{
 int cnt2,cnt3;
 double re_sym,im_sym,mag;
 cnt2=cnt1;
 Re_Tim_Corr=Im_Tim_Corr=0.0;
 for(cnt3=ZERO;cnt3<PRE_AMB_SS;cnt3++)
 {
  if(cnt2 >= RX_SAMP_SIZE)
   Error_Mes(3);
  re_sym=Map[Sym[cnt3]&ONE];
  im_sym=Im_ISI[cnt3];
  mag=re_sym*re_sym+im_sym*im_sym;
  Re_Tim_Corr=Re_Tim_Corr+(Re_MF_Op[cnt2]*re_sym+Im_MF_Op[cnt2]*im_sym)/mag;
  Im_Tim_Corr=Im_Tim_Corr+(Im_MF_Op[cnt2]*re_sym-Re_MF_Op[cnt2]*im_sym)/mag;
  cnt2=cnt2+RX_SAMP_PER_SYM;
 }
 Mean_Corr=Mean_Corr+Re_Tim_Corr*Re_Tim_Corr+Im_Tim_Corr*Im_Tim_Corr;
 Mean_Corr_Cnt++;
 return(0);
}
/******************************************************************************/
/*                 Check the timing correlation for maximum.
*******************************************************************************/
int Check_Tim_Correlation(cnt1)
int cnt1;
{
 double mag;
 mag=Re_Tim_Corr*Re_Tim_Corr+Im_Tim_Corr*Im_Tim_Corr;
 if(mag > Tim_Max_Mag)
 {
  Tim_Max_Mag=mag;
  Re_Tim_Max=Re_Tim_Corr;
  Im_Tim_Max=Im_Tim_Corr;
  Samp_Instant=cnt1;
 }
 if(Frame_Cnt == PRINT_FRAME)
  fprintf(fp4,"%lf\n",mag);
 return(0);
}
/******************************************************************************/
/*                         Compute the timing variance.
*******************************************************************************/
int Compute_Tim_Var()
{
 int tim_err;
 double correct_time,est_time;
 correct_time=(double)IDEAL_TIM_INSTANT;
 est_time=(double)Samp_Instant;
 Tim_Var_3=Tim_Var_3+(correct_time-est_time)*(correct_time-est_time);
 tim_err=abs(Samp_Instant-IDEAL_TIM_INSTANT);
 if(tim_err > Max_Tim_Err_3)
  Max_Tim_Err_3=tim_err;
 return(0);
}
/******************************************************************************/
/*                           Compute the carrier phase.
*******************************************************************************/
int Compute_Carrier_Phase()
{
 Est_Phase=atan(Im_Tim_Max/Re_Tim_Max);
 if(Re_Tim_Max < FRAC_ZERO)
  Est_Phase=Est_Phase+M_PI;
 else if((Re_Tim_Max > FRAC_ZERO) && (Im_Tim_Max < FRAC_ZERO))
  Est_Phase=Est_Phase+2.0*M_PI;
 if(IDEAL_CAR == ONE)
  Est_Phase=FRAC_ZERO;
 return(0);
}
/******************************************************************************/
/*                           Compute the amplitudes.
*******************************************************************************/
int Compute_Amplitude()
{
 double amp_err;
 Cost=cos(Est_Phase);
 Sint=sin(Est_Phase);
 Re_Amp=(Re_Tim_Max*Cost+Im_Tim_Max*Sint)/(double)PRE_AMB_SS;
 Im_Amp=(Im_Tim_Max*Cost-Re_Tim_Max*Sint)/(double)PRE_AMB_SS;
 if(IDEAL_AMP == ONE)
  Avg_Amp1=1.0;
 else
 Avg_Amp1=Re_Amp;
 amp_err=1.0-Avg_Amp1;
 Amp_RMS_Err_1=Amp_RMS_Err_1+amp_err*amp_err;
 return(0);
}
/******************************************************************************/
/*                        Compute the noise variance.
*******************************************************************************/
int Compute_Noise_Variance()
{
 int cnt2,cnt3,cnt4;
 double re_sym,im_sym,re_err,im_err;
 cnt2=Init_Compute_Noise_Variance();
 for(cnt3=ZERO;cnt3<PRE_AMB_SS;cnt3++)
 {
  cnt4=cnt2+RX_OFF;
  if(cnt4 >= RX_SAMP_SIZE)
   Error_Mes(3);
  re_sym=Map[Sym[cnt3]&ONE];
  im_sym=Map[Sym[cnt3]>>ONE];
  re_err=(Re_MF_Op[cnt2]*Cost+Im_MF_Op[cnt2]*Sint)*re_sym-Avg_Amp1;
  im_err=(Im_MF_Op[cnt4]*Cost-Re_MF_Op[cnt4]*Sint)*im_sym-Avg_Amp1;
  Est_Var=Est_Var+re_err*re_err+im_err*im_err;
  cnt2=cnt2+RX_SAMP_PER_SYM;
 }
 Get_Estimated_Noise_Variance();
 return(0);
}
/******************************************************************************/
/*      Initialize variables in the computation of the noise variance.
*******************************************************************************/
int Init_Compute_Noise_Variance()
{
 int time;
 Cost=cos(Est_Phase);
 Sint=sin(Est_Phase);
 if(IDEAL_TIM == ZERO)
  time=Samp_Instant;
 else
  time=IDEAL_TIM_INSTANT;
 Est_Var=FRAC_ZERO;
 return(time);
}
/******************************************************************************/
/*                      Get the estimated noise variance.
*******************************************************************************/
int Get_Estimated_Noise_Variance()
{
 double var_err;
 if(IDEAL_VAR == ZERO)
  Est_Var=0.5*Est_Var/(double)PRE_AMB_SS;
 else
  Est_Var=2.0*Variance;
 Inv_Twice_Noise_Var=1.0/(2.0*Est_Var);
 var_err=2.0*Variance-Est_Var;
 NVar_RMS_Err_1=NVar_RMS_Err_1+var_err*var_err;
 return(0);
}
/******************************************************************************/
/*                               Print results.
*******************************************************************************/
int Print_Sync_Results()
{
 double mag,ratio;
 mag=Mean_Corr;
 mag=mag/(double)(Mean_Corr_Cnt);
 ratio=Tim_Max_Mag/mag;
 Avg_Ratio_3=Avg_Ratio_3+ratio;
 if(ratio < Min_Ratio_3)
  Min_Ratio_3=ratio;
 printf("\t 3rd attempt.\n");
 printf("\t Time:%d.\n",Samp_Instant);
 printf("\t Mag. of peak:%lf ",Tim_Max_Mag);
 printf(" Mag. of avg:%lf.\n",mag);
 printf("\t Samples:%d Ratio:%lf\n",Mean_Corr_Cnt,ratio);
 printf("\t Est. Amp:%lf\n",Avg_Amp1);
 printf("\t Act. Var:%lf Est. Var:%lf\n\n",2.0*Variance,Est_Var);
 fflush(stdout);
 return(0);
}
/******************************************************************************/
