/******************************************************************************/
/*   Copyright (C) 2006 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "deal7type.h"
#include "deal7ext.h"
/******************************************************************************/
/*                        Estimate the frequency offset.
*******************************************************************************/
int Get_F_Off()
{
 Scan_Pre_Amb();
 if((Max_F_Off_Inx > MIN_TIM) && (Max_F_Off_Inx < MAX_TIM))
 {
  Estimate_F_Off();
  if(RAND_TIM == ZERO)
   Update_Tim_Var();
 }
 Print_F_Off();
 return(0);
}
/******************************************************************************/
/*                          Scan for preamble.
*******************************************************************************/
int Scan_Pre_Amb()
{
 int cnt1,cnt2,inx,max;
 double mag;
 Init_Scan(&cnt1,&max);
 while(cnt1 < max)
 {
  inx=cnt1;
  Re_F_Off_Sum=Im_F_Off_Sum=FRAC_ZERO;
  for(cnt2=ZERO;cnt2<RX1_PRE_AMB_SIZE;cnt2++)
  {
   if((inx+F_EST_LAG*RX_SAMP_PER_SYM) < RX_SAMP_SIZE)
    Update_F_Off_Sum(inx,cnt2);
   inx=inx+RX_SAMP_PER_SYM;
  }
  mag=Re_F_Off_Sum*Re_F_Off_Sum+Im_F_Off_Sum*Im_F_Off_Sum;
  if(Frame_Cnt == ZERO)
   fprintf(fp2,"%lf\n",mag);
  if(mag > Max_F_Off_Mag)
   Store_Max(cnt1,mag);
  cnt1++;
 }
 return(0);
}
/******************************************************************************/
/*                   Initialize scanning of the preamble.
*******************************************************************************/
int Init_Scan(cnt1_ptr,max_ptr)
int *cnt1_ptr,*max_ptr;
{
 if(Frame_Cnt == ZERO)            /* For getting the timing acquisition plot. */
 {
  *cnt1_ptr=ZERO;
  *max_ptr=3*MAX_TIM;
 }
 else
 {
  *cnt1_ptr=MIN_TIM;
  *max_ptr=MAX_TIM;
 }
 Max_F_Off_Mag=MIN_MAG;
 return(0);
}
/******************************************************************************/
/*                      Update the differential sum.
*******************************************************************************/
int Update_F_Off_Sum(inx1,cnt2)
int inx1,cnt2;
{
 int inx2,im_bit1[ISI_LEN],im_bit2[ISI_LEN];
 double re_samp1,re_samp2,im_samp1,im_samp2,re_diff,im_diff,re_sym1,re_sym2;
 double im_sym1,im_sym2;
 inx2=inx1+F_EST_LAG*RX_SAMP_PER_SYM;
 re_sym1=Map[Sym[cnt2]&ONE];
 re_sym2=Map[Sym[cnt2+F_EST_LAG]&ONE];
 im_sym1=Im_ISI[cnt2];
 im_sym2=Im_ISI[cnt2+F_EST_LAG];
 re_samp1=re_sym1*Re_MF_Op[inx1]+im_sym1*Im_MF_Op[inx1];
 im_samp1=re_sym1*Im_MF_Op[inx1]-im_sym1*Re_MF_Op[inx1];
 re_samp2=re_sym2*Re_MF_Op[inx2]+im_sym2*Im_MF_Op[inx2];
 im_samp2=re_sym2*Im_MF_Op[inx2]-im_sym2*Re_MF_Op[inx2];
 re_diff=re_samp2*re_samp1+im_samp2*im_samp1;
 im_diff=im_samp2*re_samp1-re_samp2*im_samp1;
 Re_F_Off_Sum=Re_F_Off_Sum+re_diff;
 Im_F_Off_Sum=Im_F_Off_Sum+im_diff;
 return(0);
}
/******************************************************************************/
/*              Store the maximum value of the differential sum.
*******************************************************************************/
int Store_Max(inx,mag)
int inx;
double mag;
{
 Max_F_Off_Mag=mag;
 Max_F_Off_Inx=inx;
 Max_Re_F_Off_Sum=Re_F_Off_Sum;
 Max_Im_F_Off_Sum=Im_F_Off_Sum;
 return(0);
}
/******************************************************************************/
/*                      Estimate the frequency offset.
*******************************************************************************/
int Estimate_F_Off()
{
 double theta,diff;
 if(fabs(Max_Im_F_Off_Sum) < fabs(Max_Re_F_Off_Sum))
 {
  theta=atan(Max_Im_F_Off_Sum/Max_Re_F_Off_Sum);
  if((Max_Im_F_Off_Sum > FRAC_ZERO) && (Max_Re_F_Off_Sum < FRAC_ZERO))
   theta=theta+M_PI;
  else if((Max_Im_F_Off_Sum < FRAC_ZERO) && (Max_Re_F_Off_Sum < FRAC_ZERO))
   theta=theta-M_PI;
 }
 else
 {
  theta=M_PI/2.0-atan(Max_Re_F_Off_Sum/Max_Im_F_Off_Sum);
  if(Max_Im_F_Off_Sum < FRAC_ZERO)
   theta=theta-M_PI;
 }
 Est_F_Off=theta/(double)(TX_IP_FAC*F_EST_LAG);
 diff=fabs(Est_F_Off-F_OFF);
 F_Off_RMS=F_Off_RMS+diff*diff;
 if(diff > Max_F_Off_Err)
  Max_F_Off_Err=diff;
 return(0);
}
/******************************************************************************/
/*                        Update the timing variance.
*******************************************************************************/
int Update_Tim_Var()
{
 double correct_time,est_time;
 correct_time=(double)(TX_DELAY*RX_IP_FAC*TWO);
 est_time=(double)Max_F_Off_Inx;
 Tim_Var=Tim_Var+(correct_time-est_time)*(correct_time-est_time);
 return(0);
}
/******************************************************************************/
/*                        Print the frequency offset.
*******************************************************************************/
int Print_F_Off()
{
 printf("\t Time:%d Max Re:%lf ",Max_F_Off_Inx,Max_Re_F_Off_Sum);
 printf("Max Im:%lf ",Max_Im_F_Off_Sum);
 printf("Freq Off:%lf\n",Est_F_Off);
 fflush(stdout);
 return(0);
}
/******************************************************************************/
