/******************************************************************************/
/*   Copyright (C) 2006 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "deal7type.h"
#include "deal7ext.h"
/******************************************************************************/
int Get_ML_F_Off()
{
 int cnt1,cnt2,interval;
 Init_ML_Foff();
 for(cnt1=ZERO;cnt1<MAX_ML_ITER;cnt1++)
 {
  if(cnt1 == ZERO)
   interval=ML_INTERVAL1;
  else
   interval=ML_INTERVAL2;
  for(cnt2=ZERO;cnt2<=interval;cnt2++)
  {
   Get_DFT(cnt1);
   Update_Max_Mag();
   Est_F_Off_ML=Est_F_Off_ML+ML_Step;
  }
  Update_ML_FOff_Variables(cnt1);
 }
 Update_ML_RMS_Err();
 Print_ML_Est();
 return(0);
}
/******************************************************************************/
/*          Initialize the ML frequency offset estimation procedure.
*******************************************************************************/
int Init_ML_Foff()
{
 Multiply_Sym();
 ML_Step=2.0*MAX_F_OFF_ERR/(double)(ML_INTERVAL1);
 Est_F_Off_ML=Est_F_Off_2-MAX_F_OFF_ERR;
 Max_DFT_Mag=MIN_MAG;
 return(0);
}
/******************************************************************************/
/*            Multiply with the complex conjugate of the symbols.
*******************************************************************************/
int Multiply_Sym()
{
 int cnt,inx;
 double re_sym,im_sym,re_samp,im_samp;
 if(IDEAL_TIM == 1)
  inx=TX_DELAY*RX_IP_FAC*TWO;
 else
  inx=Max_F_Off_Inx_2;
 for(cnt=ZERO;cnt<PRE_AMB_SS;cnt++)
 {
  re_sym=Map[Sym[cnt]&ONE];
  im_sym=Im_ISI[cnt];
  re_samp=Re_MF_Op[inx];
  im_samp=Im_MF_Op[inx];
  Re_MF_Op[inx]=re_samp*re_sym+im_samp*im_sym;
  Im_MF_Op[inx]=im_samp*re_sym-re_samp*im_sym;
  inx=inx+RX_SAMP_PER_SYM;
 }
 return(0);
}
/******************************************************************************/
/*                              Get the DFT.
*******************************************************************************/
int Get_DFT(iter)
int iter;
{
 int cnt,inx,length;
 double theta;
 if(IDEAL_TIM == 1)
  inx=TX_DELAY*RX_IP_FAC*TWO;
 else
  inx=Max_F_Off_Inx_2;
 if(iter == ZERO)
  length=ML_LENGTH1;
 else
  length=PRE_AMB_SS;
 theta=Re_DFT=Im_DFT=0.0;
 for(cnt=ZERO;cnt<length;cnt++)
 {
  Re_DFT=Re_DFT+Re_MF_Op[inx]*cos(theta)+Im_MF_Op[inx]*sin(theta);
  Im_DFT=Im_DFT+Im_MF_Op[inx]*cos(theta)-Re_MF_Op[inx]*sin(theta);
  theta=theta+Est_F_Off_ML*(double)TX_IP_FAC;
  if(theta > 2.0*M_PI)
   theta=theta-2.0*M_PI;
  else if(theta < -2.0*M_PI)
   theta=theta+2.0*M_PI;
  inx=inx+RX_SAMP_PER_SYM;
 }
 return(0);
}
/******************************************************************************/
/*              Update the frequency giving the max. magnitude.
*******************************************************************************/
int Update_Max_Mag()
{
 double mag;
 mag=Re_DFT*Re_DFT+Im_DFT*Im_DFT;
 if(mag > Max_DFT_Mag)
 {
  Max_DFT_Mag=mag;
  Est_F_Off_3=Est_F_Off_ML;
 }
 return(0);
}
/******************************************************************************/
/*                 Update the ML frequency offset variables.
*******************************************************************************/
int Update_ML_FOff_Variables(iter)
int iter;
{
 double tp_min,diff;
 Max_DFT_Mag=MIN_MAG;
 Est_F_Off_ML=Est_F_Off_3-ERR_RES_RATIO*ML_Step;
 ML_Step=2.0*ERR_RES_RATIO*ML_Step/(double)(ML_INTERVAL2);
 if(iter == ZERO)
 {
  diff=fabs(F_OFF-Est_F_Off-Est_F_Off_3);
  F_Off_RMS_3=F_Off_RMS_3+diff*diff;
  if(diff > Max_F_Off_Err_3)
   Max_F_Off_Err_3=diff;
 }
 return(0);
}
/******************************************************************************/
/*           Update the RMS error in the ML freq. offset estimate.
*******************************************************************************/
int Update_ML_RMS_Err()
{
 double diff;
 diff=fabs(F_OFF-Est_F_Off-Est_F_Off_3);
 F_Off_RMS_4=F_Off_RMS_4+diff*diff;
 if(diff > Max_F_Off_Err_4)
  Max_F_Off_Err_4=diff;
 return(0);
}
/******************************************************************************/
/*            Print the ML estimate of the residual freq. offset.
*******************************************************************************/
int Print_ML_Est()
{
 printf("\t Frame:%d ML foff err:%lf\n",Frame_Cnt,F_OFF-Est_F_Off-Est_F_Off_3);
 fflush(stdout);
 return(0);
}
/******************************************************************************/
