/******************************************************************************/
/*   Copyright (C) 2006 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "deal7type.h"
#include "deal7ext.h"
/******************************************************************************/
/*                             Sample the MF output.
*******************************************************************************/
int Sample_MF_Op()
{
 Preamb_Ph_Proc();
 Init_ISI_Reg_for_Ph_Est();
 Data_Process();
 if(SNR_B < TC_SNR_THRESH)
  Turbo_Dec();
 return(0);
}
/******************************************************************************/
/*                     Process the phase of the preamble.
*******************************************************************************/
int Preamb_Ph_Proc()
{
 int cnt;
 Avg_Re_Amp=Avg_Im_Amp=Avg_Amp1=Est_Var=FRAC_ZERO;
 Sym_Cnt=Amp_Ctr=Var_Ctr=ZERO;
 for(cnt=ZERO;cnt<PRE_AMB_SS;cnt++)
 {
  Est_Pre_Amb_Phase(cnt);
  if((cnt >= AMP_EST_START) && (cnt < AMP_EST_END))
   Est_Pre_Amb_Amp(cnt);
  if(cnt >= AMP_EST_END)
   Est_Noise_Var(cnt);
  Sym_Cnt=Sym_Cnt+TWO;
 }
 Get_Noise_Var();
 return(0);
}
/******************************************************************************/
/*                      Estimate the preamble phase.
*******************************************************************************/
int Est_Pre_Amb_Phase(cnt)
int cnt;
{
 double re_val,im_val,mag,re_samp,im_samp,re_tmp,im_tmp;
 re_val=Map[Sym[cnt]&ONE];
 im_val=Im_ISI[cnt];
 mag=re_val*re_val+im_val*im_val;
 re_tmp=(Re_MF_Op[Sym_Cnt]*re_val+Im_MF_Op[Sym_Cnt]*im_val)/mag;
 im_tmp=(Im_MF_Op[Sym_Cnt]*re_val-Re_MF_Op[Sym_Cnt]*im_val)/mag;
 Avg_Re_Amp=IIR_COEFF*Avg_Re_Amp+(1.0-IIR_COEFF)*re_tmp;
 Avg_Im_Amp=IIR_COEFF*Avg_Im_Amp+(1.0-IIR_COEFF)*im_tmp;
// printf("Fr:%d sym%d ",Frame_Cnt,cnt);
// printf("re:%lf im:%lf ",re_samp,im_samp);
// printf("av re:%lf av im:%lf\n",Avg_Re_Amp,Avg_Im_Amp);
 return(0);
}
/******************************************************************************/
/*                         Estimate the mean amplitude.
*******************************************************************************/
int Est_Pre_Amb_Amp(cnt)
int cnt;
{
 double amp_err;
 Derotate();
 Avg_Amp1=Avg_Amp1+Rx_Re_Sym1*Map[Sym[cnt]&ONE];
 Avg_Amp1=Avg_Amp1+Rx_Im_Sym2*Map[Sym[cnt]>>ONE];
 Amp_Ctr=Amp_Ctr+TWO;
 if(cnt == (AMP_EST_END-ONE))
 {
  if(Amp_Ctr == ZERO)
   Error_Mes(1);
  Avg_Amp1=Avg_Amp1/(double)Amp_Ctr;
  if(IDEAL_AMP == ONE)
   Avg_Amp1=1.0;
  amp_err=1.0-Avg_Amp1;
  Amp_RMS_Err=Amp_RMS_Err+amp_err*amp_err;
  printf("\t Average amplitude:%lf. Obs:%d.\n",Avg_Amp1,Amp_Ctr);
 }
 return(0);
}
/******************************************************************************/
/*                        Estimate the noise variance.
*******************************************************************************/
int Est_Noise_Var(cnt)
int cnt;
{
 double re_sym,im_sym,re_var,im_var;
 Derotate();
 re_sym=Map[Sym[cnt]&ONE];
 im_sym=Map[Sym[cnt]>>ONE];
 re_var=(Rx_Re_Sym1*re_sym-Avg_Amp1)*(Rx_Re_Sym1*re_sym-Avg_Amp1);
 im_var=(Rx_Im_Sym2*im_sym-Avg_Amp1)*(Rx_Im_Sym2*im_sym-Avg_Amp1);
 Est_Var=Est_Var+re_var+im_var;
 Var_Ctr=Var_Ctr+TWO;
 return(0);
}
/******************************************************************************/
/*                        Get the noise variance.
*******************************************************************************/
int Get_Noise_Var()
{
 double var_err;
 if(Var_Ctr == ZERO)
  Error_Mes(1);
 Est_Var=Est_Var/(double)Var_Ctr;
 if(IDEAL_VAR == ZERO)
  Inv_Twice_Noise_Var=1.0/(2.0*Est_Var);
 printf("\t Act. Var:%lf Est. Var:%lf Obs:%d\n",2.0*Variance,Est_Var,Var_Ctr);
 var_err=2.0*Variance-Est_Var;
 NVar_RMS_Err=NVar_RMS_Err+var_err*var_err;
 return(0);
}
/******************************************************************************/
/* Initialize the ISI reg. for phase estimation. The most recent symbol is
 * at zero.
*******************************************************************************/
int Init_ISI_Reg_for_Ph_Est()
{
 int cnt;
 for(cnt=ZERO;cnt<ISI_LEN;cnt++)
  ISI_Reg[cnt]=Map[Sym[PRE_AMB_SIZE-ONE-cnt]>>ONE];
 return(0);
}
/******************************************************************************/
/*                  Process the phase of the data symbols.
*******************************************************************************/
int Data_Process()
{
 int cnt;
 for(cnt=ZERO;cnt<(SYM_SIZE+SYM_BUF_LEN);cnt++)
 {
  Derotate();
  if((cnt >= SYM_BUF_LEN) && (cnt < (SYM_BUF_LEN+DATA_SIZE)))
  {
   Get_Errors(cnt);
   Get_Noisy_Sym(cnt);
  }
  Shift_Into_ISI_Reg();
  Update_Data_Phase();
  Sym_Cnt=Sym_Cnt+TWO;
 }
 return(0);
}
/******************************************************************************/
/*                  Derotate samples to get current symbol.
*******************************************************************************/
int Derotate()
{
 double re_tmp1,im_tmp1,re_tmp2,im_tmp2;
 if((Sym_Cnt+ONE) >= RX_SAMP_SIZE)
  Error_Mes(3);
 re_tmp1=Re_MF_Op[Sym_Cnt];
 im_tmp1=Im_MF_Op[Sym_Cnt];
 re_tmp2=Re_MF_Op[Sym_Cnt+ONE];
 im_tmp2=Im_MF_Op[Sym_Cnt+ONE];
 Est_Phase=atan(Avg_Im_Amp/Avg_Re_Amp);
 if(Avg_Re_Amp < FRAC_ZERO)
  Est_Phase=Est_Phase+M_PI;
 else if((Avg_Re_Amp > FRAC_ZERO) && (Avg_Im_Amp < FRAC_ZERO))
  Est_Phase=Est_Phase+2.0*M_PI;
 if(IDEAL_CAR == 1)
  Est_Phase=0.0;
 Cost=cos(Est_Phase);
 Sint=sin(Est_Phase);
 Rx_Re_Sym1=re_tmp1*Cost+im_tmp1*Sint;
 Rx_Im_Sym1=im_tmp1*Cost-re_tmp1*Sint;
 Rx_Re_Sym2=re_tmp2*Cost+im_tmp2*Sint;
 Rx_Im_Sym2=im_tmp2*Cost-re_tmp2*Sint;
// printf("ti:%d rxre:%lf rxim:%lf ",Sym_Cnt,Rx_Re_Sym1,Rx_Im_Sym2);
// printf("txre:%d ",Sym[cnt+PRE_AMB_SIZE-SYM_BUF_LEN]&1);
// printf("txim:%d\n",Sym[cnt+PRE_AMB_SIZE-SYM_BUF_LEN]>>1);
 return(0);
}
/******************************************************************************/
/*                              Get the errors.
*******************************************************************************/
int Get_Errors(cnt)
int cnt;
{
 int bit0,bit1,rx_bit0,rx_bit1,cnt1;
 rx_bit0=rx_bit1=ZERO;
 bit0=Sym[cnt+PRE_AMB_SIZE-SYM_BUF_LEN]&ONE;
 bit1=Sym[cnt+PRE_AMB_SIZE-SYM_BUF_LEN]>>ONE;
 if(Rx_Re_Sym1 < FRAC_ZERO)
  rx_bit0=ONE;
 if(Rx_Im_Sym2 < FRAC_ZERO)
  rx_bit1=ONE;
 if(bit0 != rx_bit0)
  Ber++;
 if(bit1 != rx_bit1)
  Ber++;
 return(0);
}
/******************************************************************************/
/*                          Get the noisy symbols.
*******************************************************************************/
int Get_Noisy_Sym(cnt)
int cnt;
{
 int inx;
 Sym0[cnt-SYM_BUF_LEN]=Rx_Re_Sym1;
 if(((cnt-SYM_BUF_LEN)%TWO) == ZERO)
  Sym1[cnt-SYM_BUF_LEN]=Rx_Im_Sym2;
 else
  Sym2[cnt-SYM_BUF_LEN]=Rx_Im_Sym2;
// printf("time:%d Sym0:%lf\n",cnt-SYM_BUF_LEN,Sym0[cnt-SYM_BUF_LEN]);
 return(0);
}
/******************************************************************************/
/*  Shift new data into the ISI registers. The most recent symbol is at zero.
 *  The symbol is obtained two symbol instants from future time.
*******************************************************************************/
int Shift_Into_ISI_Reg()
{
 int cnt,inx;
 double re_tmp1,im_tmp1,re_sym3,im_sym3;
 for(cnt=ISI_LEN-ONE;cnt>ZERO;cnt--)
  ISI_Reg[cnt]=ISI_Reg[cnt-ONE];
 inx=Sym_Cnt+TWO*SYM_BUF_LEN+ONE;
 if(inx >= RX_SAMP_SIZE)
  Error_Mes(3);
 re_tmp1=Re_MF_Op[inx];
 im_tmp1=Im_MF_Op[inx];
 re_sym3=re_tmp1*Cost+im_tmp1*Sint;
 im_sym3=im_tmp1*Cost-re_tmp1*Sint;
 if(im_sym3 > FRAC_ZERO)
  ISI_Reg[ZERO]=  1.0;
 else
  ISI_Reg[ZERO]= -1.0;
 return(0);
}
/******************************************************************************/
/*               Update the carrier phase of the data symbols.
*******************************************************************************/
int Update_Data_Phase()
{
 int cnt;
 double re_val,im_val,mag,re_tmp,im_tmp;
 im_val=0.0;
 for(cnt=ZERO;cnt<ISI_LEN;cnt++)
  im_val=im_val+ISI_Coeff[cnt]*ISI_Reg[cnt];
 if(Rx_Re_Sym1 > FRAC_ZERO)
  re_val=  1.0;
 else
  re_val= -1.0;
 mag=re_val*re_val+im_val*im_val;
 re_tmp=(Re_MF_Op[Sym_Cnt]*re_val+Im_MF_Op[Sym_Cnt]*im_val)/mag;
 im_tmp=(Im_MF_Op[Sym_Cnt]*re_val-Re_MF_Op[Sym_Cnt]*im_val)/mag;
 Avg_Re_Amp=IIR_COEFF*Avg_Re_Amp+(1.0-IIR_COEFF)*re_tmp;
 Avg_Im_Amp=IIR_COEFF*Avg_Im_Amp+(1.0-IIR_COEFF)*im_tmp;
// printf("data Fr:%d ti:%d re sym:%lf ",Frame_Cnt,Sym_Cnt,re_val);
// printf("imisi:%lf\n",im_val);
// printf("re:%lf im:%lf ",re_tmp,im_tmp);
// printf("avre:%lf avim:%lf\n",Avg_Re_Amp,Avg_Im_Amp);
 return(0);
}
/******************************************************************************/
/*                         Turbo decoder routines.
*******************************************************************************/
int Turbo_Dec()
{
 Init_Gamma_I_E();
 Init_App();
 Bcjr();
 Get_Bit_Errors_Dec1();
 return(0);
}
/******************************************************************************/
/*          Initialize the intrinsic and extrinsic values of gamma.
*******************************************************************************/
int Init_Gamma_I_E()
{
 Init_Gamma_I_E_Dec1();
 Init_Gamma_I_E_Dec2();
 return(0);
}
/******************************************************************************/
/*    Initialize the intrinsic and extrinsic values of gamma for decoder 1.
*******************************************************************************/
int Init_Gamma_I_E_Dec1()
{
 int time,bit;
 for(time=ZERO;time<DATA_SIZE;time++)
 {
  for(bit=ZERO;bit<NUM_INPUTS;bit++)
  {
   Compute_Gamma_E_P1(bit,time);
   Compute_Gamma_I(bit,time);
  }
 }
 return(0);
}
/******************************************************************************/
/*     Compute gamma from parity bit (extrinsic information) for decoder 1.
*******************************************************************************/
int Compute_Gamma_E_P1(bit,time)
int bit,time;
{
 double sym,x;
 if((time%TWO) == ZERO)
 {
  sym=Map[bit]*Avg_Amp1;
  x=sym-Sym1[time];
  x= -x*x*Inv_Twice_Noise_Var;
  Gamma_E_P1[time][bit]=exp(x);
 }
 else
  Gamma_E_P1[time][bit]=1.0;
 return(0);
}
/******************************************************************************/
/*      Compute gamma from data bit (intrinsic information) for decoder 1.
*******************************************************************************/
int Compute_Gamma_I(bit,time)
int bit,time;
{
 double sym,x;
 sym=Map[bit]*Avg_Amp1;
 x=sym-Sym0[time];
 x= -x*x*Inv_Twice_Noise_Var;
 Gamma_I[time][bit]=exp(x);
 return(0);
}
/******************************************************************************/
/*    Initialize the intrinsic and extrinsic values of gamma for decoder 2.
*******************************************************************************/
int Init_Gamma_I_E_Dec2()
{
 int time,bit;
 for(time=ZERO;time<DATA_SIZE;time++)
 {
  for(bit=ZERO;bit<NUM_INPUTS;bit++)
   Compute_Gamma_E_P2(bit,time);
 }
 return(0);
}
/******************************************************************************/
/*     Compute gamma from parity bit (extrinsic information) for decoder 2.
*******************************************************************************/
int Compute_Gamma_E_P2(bit,time)
int bit,time;
{
 double sym,x;
 if((time%TWO) != ZERO)
 {
  sym=Map[bit]*Avg_Amp1;
  x=sym-Sym2[time];
  x= -x*x*Inv_Twice_Noise_Var;
  Gamma_E_P2[time][bit]=exp(x);
 }
 else
  Gamma_E_P2[time][bit]=1.0;
 return(0);
}
/******************************************************************************/
/*                Initialize the a posteriori probabilities.
*******************************************************************************/
int Init_App()
{
 int time,input;
 for(time=ZERO;time<DATA_SIZE;time++)
 {
  for(input=ZERO;input<NUM_INPUTS;input++)
   Prob[time][input]=0.5;
 }
 return(0);
}
/******************************************************************************/
/*                          The BCJR algorithm.
*******************************************************************************/
int Bcjr()
{
 for(Iter=ZERO;Iter<MAX_ITER;Iter++)
 {
  Dec2_Process();
  Dec1_Process();
 }
 return(0);
}
/******************************************************************************/
/*    Finally compute the number of bit errors in the frame from decoder1.
*******************************************************************************/
int Get_Bit_Errors_Dec1()
{
 int time,detected_bit,act_bit;
 for(time=ZERO;time<DATA_SIZE;time++)
 {
  detected_bit=ONE;
  if(Prob[time][ZERO] > Prob[time][ONE])
   detected_bit=ZERO;
  act_bit=Bits[time];
  if(detected_bit != Bits[time])
   T_Ber++;
  Valid_Bit++;
 }
 return(0);
}
/******************************************************************************/
/*                       Print out the probabilities.
*******************************************************************************/
int Print_Prob()
{
 int time,act_bit;
 for(time=ZERO;time<DATA_SIZE;time++)
 {
  act_bit=Bits[time];
  printf("Print prob Frame:%d time:%d tx bit:%d ",Frame_Cnt,time,act_bit);
  printf("Prob 0:%lf  ",Prob[time][0]);
  printf("Prob 1:%lf\n",Prob[time][1]);
 }
 return(0);
}
/******************************************************************************/
