/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
/******************************************************************************/
/*                         PARAMETERS RELATED TO A FRAME.
*******************************************************************************/
#define PRE_AMB_SIZE 2000                              /* Length of preamble. */
#define RX1_PRE_AMB_SIZE (PRE_AMB_SIZE/2)      /* Length of preamble for rx1. */
#define PRE_AMB_SS (PRE_AMB_SIZE-SYM_BUF_LEN)       /* Steady-state preamble. */
#define DATA_SIZE 20000                               /* Length of data bits. */
#define DATA_SIZE_P1 (DATA_SIZE+ONE)                    /* Datasize plus one. */
#define SYM_SIZE DATA_SIZE                         /* Length of data symbols. */
#define POST_AMBLE 12                                      /* Postamble size. */
#define FRAME_SIZE (PRE_AMB_SIZE+SYM_SIZE+POST_AMBLE)           /* Framesize. */
#define MAX_FRAMES 500                      /* No. of frames to be simulated. */
#define AMP_EST_START (PRE_AMB_SS/TWO)      /* Start of amplitude estimation. */
#define AMP_EST_END (THREE*PRE_AMB_SS/FOUR)   /* End of amplitude estimation. */
#define BPSK_SIZE 2                                    /* BPSK constellation. */
#define IV_SIZE DATA_SIZE                         /* Size of the interleaver. */
/******************************************************************************/
/*                          TRELLIS PARAMETERS.
*******************************************************************************/
#define MEM_SIZE 4                                      /* Memory of encoder. */
#define ENC_SIZE (MEM_SIZE+1)                /* Max. no. of taps for encoder. */
#define NUM_STATES 16              /* No. of states in the component trellis. */
#define NUM_INPUTS 2                          /* No. of inputs to each state. */
/******************************************************************************/
/*                         TRANSMIT FILTER PARAMETERS.
*******************************************************************************/
#define TX_DELAY 12                                 /* Transmit filter delay. */
#define TX_FLT_SIZE (TWO*TX_DELAY+ONE)                    /* Size of tx. flt. */
#define TX_OFF (TX_IP_FAC/TWO)                               /* Offset at tx. */
#define RX_OFF (TX_OFF*RX_IP_FAC)                            /* Offset at rx. */
#define TX_FLT_SIZE_1 (TX_FLT_SIZE+TX_OFF)            /* Size of im. tx. reg. */
#define SYMBOL_RATE 1.0                                       /* Symbol rate. */
#define TX_IP_FAC 4                            /* Interpolation factor at tx. */
#define TX_SAMP_FREQ ((double)TX_IP_FAC*SYMBOL_RATE)    /* Samp. freq. at tx. */
#define ROLL_OFF 0.41                                            /* Roll-off. */
/******************************************************************************/
/*                         FREQUENCY OFFSET PARAMETERS.
*******************************************************************************/
#define F_OFF (+0.15*M_PI)                        /* Freq. offset in radians. */
#define F_EST_LAG 1       /* Lag for 1st and 2nd attempts for freq. off. est. */
#define ISI_LEN 6                                /* Length of ISI in Im. arm. */
#define SYM_BUF_LEN ((ISI_LEN/2)-ONE)             /* To store future symbols. */
#define MAX_ML_ITER 2            /* No. of iter. for ML est. of freq. offset. */
#define ML_INTERVAL1 1600                                /* No. of intervals. */
#define ML_INTERVAL2 200                                 /* No. of intervals. */
#define ML_LENGTH1 200                    /* Length of DFT in 1st ML attempt. */
#define MAX_F_OFF_ERR 0.100      /* Max. freq. off. est error in 2nd attempt. */
#define ERR_RES_RATIO 8.0       /* Max err by res. in ML 1st stage (8 or 16). */
#define IIR_COEFF 0.995                   /* IIR filter coeff. for phase avg. */
/******************************************************************************/
/*                         MATCHED FILTER PARAMETERS.
*******************************************************************************/
#define RX_IP_FAC 4                            /* Interpolation factor at rx. */
#define MF_DELAY (TX_DELAY*RX_IP_FAC)         /* Delay of the matched filter. */
#define MF_SIZE (TWO*MF_DELAY+ONE)                    /* Matched filter size. */
#define MF_AC_SIZE (TWO*MF_SIZE-ONE)                     /* MF autocor. size. */
#define RX_SAMP_FREQ (TX_SAMP_FREQ*(double)RX_IP_FAC)       /* MF samp. freq. */
#define RX_SAMP_SIZE (2*PRE_AMB_SIZE*RX_SAMP_PER_SYM)         /* Rx. samples. */
#define TX_SAMP_SIZE (FRAME_SIZE*TX_IP_FAC)           /* Tx. samp. per frame. */
#define RX_SAMP_PER_SYM (TX_IP_FAC*RX_IP_FAC)         /* Rx. samples per sym. */
/******************************************************************************/
/*                  GAUSSIAN NOISE GENERATOR PARAMETERS.
*******************************************************************************/
#define RAND_PHASE 0                        /* 1: Random phase; 0: otherwise. */
#define RAND_TIM 0                         /* 1: Random timing; 0: otherwise. */
#define IDEAL_TIM 0                         /* 1: Ideal timing; 0: otherwise. */
#define IDEAL_CAR 0                        /* 1: Ideal carrier; 0: otherwise. */
#define SNR_B (+1.00)                                       /* SNR per bit. */
#define MAX_SNR 100.0
#define TC_SNR_THRESH 3.0                /* SNR threshold for turbo decoding. */
#define PN_SIZE 3                 /* Size of array which stores the PN value. */
#define NVALSIZE 4
#define URADIUS 0
#define UTHETA 1
#define GRAND_0 2
#define GRAND_1 3
/******************************************************************************/
/*                        LOCAL OSCILLATOR PARAMETERS.
*******************************************************************************/
#define LO_PERIOD 4                                             /* LO period. */
/******************************************************************************/
/*                            TIMING PARAMETERS.
*******************************************************************************/
#define MIN_TIM (TWO*TX_DELAY*RX_IP_FAC-RX_SAMP_PER_SYM)         /* Min. tim. */
#define MAX_TIM (TWO*TX_DELAY*RX_IP_FAC+TWO*RX_SAMP_PER_SYM)     /* Max. tim. */
/******************************************************************************/
/*                             BCJR PARAMETERS.
*******************************************************************************/
#define MAX_ITER 8                                      /* No. of iterations. */
#define IDEAL_AMP 0                      /* 1: Amplitude known; 0: otherwise. */
#define IDEAL_VAR 0                 /* 1: Noise variance known; 0: otherwise. */
/******************************************************************************/
/*                                PRINT TIME.
*******************************************************************************/
#define PRINT_TIME 1
#define PRINT_FRAME 250
/******************************************************************************/
/*                                CONSTANTS.
*******************************************************************************/
#define BINSIZE 16         /* Size of array to store states in binary format. */
#define FOUR 4
#define THREE 3
#define TWO 2
#define ONE 1
#define ZERO 0
#define TRUE 1
#define FALSE 0
#define LEFT 0
#define RIGHT 1
#define RE 0
#define IM 1
#define MINUSONE (-1)
#define ERROR 0.000001
#define FRAC_ZERO 0.0
#define MIN_MAG (-10.0)
#define MY_PI 3.14159265358979323846
#define PIBY2 1.57079632679489661923
#define PIBY4 0.785398163397448309615
#define TWICE_PI 6.28318530717958647692
/******************************************************************************/
